#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#ifndef WINDOWS
#include <pthread.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <signal.h>
#include <netdb.h>
#ifndef NO_CRYPT_DOT_H
#include <crypt.h>
#endif
#endif
#include <errno.h>
#include <time.h>
#include <ctype.h>
#ifdef WINDOWS
#include <windows.h>
#endif

#include "globals.h"
#include "functions.h"
#include "language.h"

void safestrncpy(char *dest, char *source, int n)
{
int r;

  for (r=0; r<n; r++)
  { 
    dest[r]=source[r]; 
    if (source[r]==0) break;
  }  

  dest[r]=0;
}

int socketdie(int socketid)
{
int i;

  i=shutdown(socketid,SHUT_RDWR);
#ifdef WINDOWS
  closesocket(socketid);
#else
  close(socketid);
#endif

  return i;
}

int logfile_on(int ID)
{
  logging=1;
  logfp=fopen(logfile,"a");
  if (logfp==0)
  {
    if (ID!=-1)
    { message(ID,CHAT_LOG_CANT_OPEN,0); }
      else
    { printf("Cannot open chat log for writing!\n"); }

    logging=0;
  }

  return 0;
}

#ifdef LEET

/* 13374 patch donated by Oliver Hillmann (tequila :) */

void leet(unsigned char *buf)
{
char lame_dict[] = "elaszotg";
char leet_dict[] = "314z2076";
int r, i, j, l;

  l=strlen(lame_dict);
  r=strlen(buf);

  for (i = 0; i < r; i++) 
  {

    buf[i] = (char)tolower((int)buf[i]);

    for (j = 0; j < l; j++)
    {
      if (buf[i] == lame_dict[j])
      {
        buf[i] = leet_dict[j];
        break;
      }
    }
  }    
}

#endif

char *strcasestr_m(char *hejstack, char *needle)
{
int hejlen,neelen;
int t,r;

  hejlen=strlen(hejstack);
  neelen=strlen(needle);
  r=hejlen-neelen;

  for(t=0; t<=r; t++)
  {
    if (strncasecmp(&hejstack[t],needle,neelen)==0)
    { return &hejstack[t]; }
  }

  return (char *)0;
}

void destroy()
{

  if (logfp!=0) fclose(logfp);

#ifdef DEBUG
  if (debug==1)
  { printf("Chat server terminated.\n"); }
#endif

#ifndef WINDOWS
  kill(0,SIGKILL);
#endif
  exit(0);
}

#ifndef WINDOWS
void broken_pipe(int signal_num)
{
#ifdef DEBUG
  if (debug==1) printf("Signal thrown %d\n",signal_num);
#endif

  setsignals();
}

void setsignals()
{
  signal(SIGPIPE,broken_pipe);
  signal(SIGURG,broken_pipe);
  signal(SIGIO,broken_pipe);
  signal(SIGHUP,broken_pipe);

  signal(SIGINT,destroy);
  signal(SIGTERM,destroy);
}
#endif

char *collapse(char *d, int c, char *old)
{
char *s;

  s=d;
  while(s[0]!=0) s++;
  s++;
  while(1)
  {
    if (c<=0) break;
    d[0]=s[0];
    if (d[0]==0) c--;
    d++; s++;
  }
  d++;
  strcpy(d,old);
  return d;
}

char *unban_number(int i)
{
char *banptr;
char temp[256];
int r;

  banptr=bans;
  for (r=0; r<bancount; r++)
  {
    if (r==i)
    {
      bancount--;
      strcpy(temp,banptr);
      return collapse(banptr,bancount-i,temp);
    }

    banptr=banptr+strlen(banptr)+1;
  }

  return 0;
}

int ban_address(char *addr)
{
char *banptr;
int r;

  banptr=bans;
  for (r=0; r<bancount; r++)
  {
    while(banptr[0]!=0) banptr++;
    banptr++;
  }

  if (banptr-bans+strlen(addr)+1>BANS_BUFFER_SIZE)
  { return 1; }
  strcpy(banptr,addr);
  bancount++;

  return 0;
}

int get_num(char *s, int ID, int flags)
{
int r,n=0;

  r=0;
  while (s[r]!=0 && (s[r]>='0' && s[r]<='9'))
  {
    if (r<8) n=n*10+(s[r]-'0');
    r++;
  }

  if (r==0) 
  {
    if ((flags&CAN_BE_NOTHING)==0)
    { message(ID,UNDERSTAND_MESSAGE,0); }
    return -1;
  }

  if ((flags&NOT_USERID)==0)
  {
    if (n>=maxconn || users[n]==0 || users[n]->inuse!=1)
    {
      if (ID>=0) message(ID,NOT_HERE,0);
      return -2;
    }
  }

  return n;
}

int strip(char *s)
{
int r=0;

   while(s[r]>='0' && s[r]<='9') r++;
   while(s[r]==' ') r++;

   return r;
}

void timediff(time_t time1, time_t time2, char *thisString)
{
int temp;
char tempString[70];

  thisString[0]=0;
  if (time1<time2) { temp=time1; time1=time2; time2=temp; }
  temp=time1-time2;
  if (temp!=0) 
  { sprintf(tempString,"%dsec",temp%60); temp=temp/60; strcpy(thisString,tempString); }
  if (temp!=0)
  { sprintf(tempString,"%dmin %s",temp%60,thisString); temp=temp/60; strcpy(thisString,tempString); }
  if (temp!=0)
  { sprintf(tempString,"%dhours %s",temp%24,thisString); temp=temp/24; strcpy(thisString,tempString); }
  if (temp!=0)
  { sprintf(tempString,"%ddays %s",temp%365,thisString); temp=temp/365; strcpy(thisString,tempString); }
  if (temp!=0)
  { sprintf(tempString,"%dyears %s",temp,thisString); strcpy(thisString,tempString); }
}

void shorttimediff(time_t time1, time_t time2, char *thisString)
{
int temp;

  thisString[0]=0;
  if (time1<time2) { temp=time1; time1=time2; time2=temp; }
  temp=time1-time2;
  if (temp!=0) { sprintf(thisString,"%ds",temp%60); temp=temp/60; }
  if (temp!=0) { sprintf(thisString,"%dm",temp%60); temp=temp/60; }
  if (temp!=0) { sprintf(thisString,"%dh",temp%24); temp=temp/24; }
  if (temp!=0) { sprintf(thisString,"%dd",temp%365); temp=temp/365; }
  if (temp!=0) { sprintf(thisString,"%dy",temp); }
}

int channel(int ID, char *oName)
{
int r,new_channel;
char Name[CHANNEL_NAME_LEN];

  safestrncpy(Name,oName,CHANNEL_NAME_LEN-1);

#ifdef HIDDEN_CAVES
  if (strcmp(Name,"Hidden Cave")==0) sprintf(Name,"1%3d",(int)(time(NULL)%1000));
#endif

  /* if (strcmp(Name,"0")==0 || strcasecmp(Name,MAINCHANNEL)==0) */

  if (strcmp(Name,"0")==0) 
  { strcpy(Name,MAINCHANNEL); }

  /* Check if user is already in this channel */
  if (strcasecmp(Name,channels[users[ID]->channel])==0) return -2;

  /* Check if the channel is locked */
/*
  for (r=1; r<MAXCHAN; r++)
    if ((strcasecmp(Name,channels[r])==0 && (cflags[r]&1)==1) && users[ID]->user_level<10) return -1;
*/

  new_channel=-1;
  /* for (r=0; r<=MAXCHAN; r++) */

  /* Check if channel exists already */

  for (r=0; r<MAXCHAN; r++)
  { 
    if (channels[r][0]==0) new_channel=-2;
    if (strcasecmp(Name,channels[r])==0)
    {
      /* If channel exists and is locked, don't let user in */
      if ((cflags[r]&1)==1 && users[ID]->user_level<10) return -1;
      new_channel=r;
      break;
    }
  }
   
  if (users[ID]->channel!=0 && new_channel!=-1)
  {
    for (r=0; r<maxconn; r++)
    {
      if (users[r]->inuse==1)
      {
        if (users[r]->channel==users[ID]->channel && r!=ID) r=maxconn;
      }
    }

    /* if (r<MAXCHAN+2) */
    if (r==maxconn)
    {
      channels[users[ID]->channel][0]=0;
      channelop[users[ID]->channel]=-1;
      cflags[users[ID]->channel]=0;
    }
  }

  if (new_channel>=0) return new_channel;
    else
  if (new_channel==-1) return -3;
       
  /* for (r=1; r<=MAXCHAN; r++) */
  for (r=1; r<MAXCHAN; r++)
  {
    if (channels[r][0]==0)
    {
      strcpy(channels[r],Name);
      if ((serverflags&1)==0)
      { channelop[r]=ID; }
        else
      { channelop[r]=-1; }
      return r;
    }
  }

  return users[ID]->channel;
}

int conv_num(unsigned char *s)
{
int r,n;

  n=0; r=0;

  while(r<10)
  {
    if (s[r]<'0' || s[r]>'9') break;
    n=(n*10)+(s[r]-'0');
    r++;
  }
  if (r==0) return -1;

  return n;
}

int is_number(char *myString)
{
int r=0;

  while((myString[r]!=' ' && myString[r]!='\t') &&
	  ((myString[r]!='\n' && myString[r]!='\r') && myString[r]!=0))
  {
    if (myString[r]<'0' || myString[r]>'9') return 0;
    r++;
  }

  if (r==0) return 0;
  return 1;
}

